/* UMN's wonderful Gopher Server */
/* Coded by Chris McNeil, Mount Allison University */
/* cmcneil@mta.ca  */
/* Parts of code taken from Fingerd.c  */

/*
   Modified by Peter Crawshaw (pcrawshaw@mta.ca) July 1993.
   Computing Services
   Mount Allison University
   Sackville, NB E0A 3C0

   Text file searches and dBase .dbf file searches added.
*/

/*
   Modified by Ashok Aiyar (axa12@po.cwru.edu) December 1993, January 1994.
   Dept. of Biochemistry,
   CWRU School of Medicine
   Cleveland, Ohio 44106

   Added "concatenated" index file searches that return menu items
   that match a pattern.  Imposed length restrictions on all searches.
   Additionally certain defined strings can be eliminated from
   directory searches, menu-file searches and DBF file searches.
   mkginfo function redone.  mkginfo now supports pre-made indexes too.
*/

/* Copyright 1991 Phil Karn, KA9Q */

#define CSODATABASE "CSO.DBF"
#define WORDSFILE "BADWORDS.TXT"
#define DIRBAD "GOPHER.BAD"

#define GOPHER_PORT 70

#include <stdio.h>
#include <string.h>
#include <time.h>
#include <dir.h>
#include <dos.h>
#include "global.h"
#include "files.h"
#include "mbuf.h"
#include "socket.h"
#include "session.h"
#include "proc.h"
#include "dirutil.h"
#include "commands.h"
#include "mailbox.h"
#include "netuser.h"
#include "ftp.h"
#include "alloc.h"
#include "dbf.h"

#define GOPHER_LOG "gopher.log"
#define RO_SUBDIR 0
#define RO_CASE   0
#define TRUE 1
#define FALSE 0

static int Sgopher = -1;        /* Prototype socket for service */

char *dummy = "dummy";

char *Day[7] = { "Sunday","Monday","Tuesday","Wednesday",
		"Thursday","Friday","Saturday" };
char *Month[12] = { "January","February","March","April","May","June",
		"July","August","September","October","November","December" };
char *ftime __ARGS((long *t));
char *City(void);
char *Message(void);
int GophUsers;

/* function prototypes */
int dirmatchmake(char *pattern);
int matchmake(char *pattern);
int search(char *filename, char *pattern);
int fsplit(char *P, char *D, char *F);
int gnamefix(char *s);
int DoInfo(int s, char *table);
char *TheTime(void);
int searchdirfile(int s, char *file, char *pattern);
int searchdir(int s, char *dir, char *pattern);
static void gopher __ARGS((int s,void *unused,void *p));
int split(char *input, int *type, char *file, char *pattern);
int slashfix(char *s);
int revslashfix(char *s);
static int gopherlog __ARGS((char *fmt,...));
int mail_logfile();
int sendgopherlog __ARGS((int argc, char *argv[], void *p));
int mkginfo(int s,char *file);

/* Start up gopher service */
int gstart(int argc, char *argv[], void *p)
{
	struct sockaddr_in lsocket;
	int s;

	if(Sgopher != -1){
		return 0;
	}
	psignal(Curproc,0);     /* Don't keep the parser waiting */
	chname(Curproc,"Gopher Server");

	lsocket.sin_family = AF_INET;
	lsocket.sin_addr.s_addr = INADDR_ANY;
	if(argc < 2)
		lsocket.sin_port = IPPORT_GOPHER;
	else
		lsocket.sin_port = atoi(argv[1]);

	Sgopher = socket(AF_INET,SOCK_STREAM,0);
	bind(Sgopher,(char *)&lsocket,sizeof(lsocket));
	listen(Sgopher,1);

	gopherlog("Gopher server started\n");
	for(;;)
	{
		if((s = accept(Sgopher,NULLCHAR,(int *)NULL)) == -1)
			break;  /* Service is shutting down */

		/* Spawn a server */
		newproc("gopher",1024,gopher,s,NULL,NULL,0);
	}
	return 0;
}


static void gopher(int s, void *unused, void *p)
{
	struct sockaddr fsocket;
	char command[100], *cp;
	long t;
	FILE *fp;
	char file[150];
	char *dirfile;
	char *dir, *pattern;
	int length;
	char type[1];
	int i, tt;
	int transfer; /* Transfer type, either ascii or image */
	file[0] = '\0';

	sockmode(s, SOCK_ASCII);
	sockowner(s, Curproc);
	log(s, "open Gopher");
	GophUsers++;

	recvline(s, command, 80);
	rip(command);
	length = strlen(command);
	command[length] = '\0';
	type[0] = command[0];

	gopherlog("%s ", TheTime());

	/* who connected */
	i = SOCKSIZE;
	if (getpeername(s, (char *)&fsocket, &i) != -1)
		gopherlog(" %s", psocket(&fsocket));

	/* type of service */
	if (length != 0)
		gopherlog(" %s", command);

	gopherlog("\n");
	/* if blank line was received or it is a + */
	if((length == 0) || (!strcmpi("\t+",command)) ||
	  (!strcmpi("/",command)) || (!strcmpi("\t$",command)) ||
	  (!strcmpi("\t!",command)))
	{
		strcat(file, Gopherdir);
		strcat(file, "/ginfo");
	}
	else if (!strncmp(command, "1/", 2))
		 {
		strcat(file, Gopherdir);
		strcat(file, "/ginfo");
		fp = fopen(file, READ_TEXT);
		if (fp != NULLFILE)
		{
		 sendfile(fp, s, transfer, 0, NULL);
		 fclose(fp);
		}
		 }
	else if (!strncmp(command, "time", 4))
		 {
		time(&t);
		usprintf(s, "\nCurrent date & time at %s:\n%s\n\n%s\n.\012",
			City(), ftime(&t), Message());
		goto quit;
		 }
	else if (!strncmp(command, "fields", 6))
		 {
		transfer = IMAGE_TYPE;
		sockmode(s, SOCK_BINARY);
		dbf_fields(s, CSODATABASE, CSOTEXT);
		usprintf(s, "200:Ok.\n");
		type[0] = '2';
		 }
	else if (!strncmp(command, "query", 5))
		 {
		transfer = IMAGE_TYPE;
		sockmode(s, SOCK_BINARY);
		for (i=0; i < strlen(command); i++)
		   if (command[i] == '"')
			   command[i] = ' ';
		cp = command;
		while (*cp != ' ') cp++;
		dbf_do_search(s,++cp, CSOTEXT,CSODATABASE);
		type[0] = '2';
		 }
	else if (!strncmp(command, "ph", 2))
		 {
		transfer = IMAGE_TYPE;
		sockmode(s, SOCK_BINARY);
		for (i=0; i < strlen(command); i++)
		if (command[i] == '"')
			   command[i] = ' ';
		cp = command;
		while (*cp != ' ') cp++;
		dbf_do_search(s,++cp, CSOTEXT,CSODATABASE);
		type[0] = '2';
		 }
	else {
	/* copy command to file name, delete any .. to avoid */
	/* anyone telnetting to gopher and cd'ing to a directory */
	/* other than drive:\gopher\
					*/
		file[0] = command[1];
		file[1] = ':';
		file[2] = '\0';
		strcat(file, Gopherdir);
		for ( i=3; i <= strlen(command); i++ ) {
			if (command[i] == '.' && command[i+1] == '.') {
				command[i] = ' ';
				command[i+1] = ' ';
			}
					file[(strlen(Gopherdir)+i-1)] = command[i];
			}

		}


	switch (type[0]) {
		case '0':
			transfer = ASCII_TYPE;
			break;
		case '1':
			transfer = ASCII_TYPE;
			strcat(file, "/ginfo");
			break;
		case '2':
		case '3':
		case '4':
			transfer = ASCII_TYPE;
			break;
		case '5':
			transfer = IMAGE_TYPE;
			break;
		case '6':
			transfer = ASCII_TYPE;
			break;
		case '7':
			transfer = ASCII_TYPE;
			sockmode(s, SOCK_ASCII);
			dir = strtok(file, "\t");
			pattern = strtok(NULL, "\t");
			searchdir(s, dir, pattern);
			break;
		case '8':
		case '9':
			transfer = IMAGE_TYPE;
			break;
		case 's':
			transfer = ASCII_TYPE;
			sockmode(s, SOCK_ASCII);
			dirfile = strtok(file,"\t");
			pattern = strtok(NULL, "\t");
			searchdirfile(s, file, pattern);
			break;
		/* send a listing of the fields in a dbf file */
		case 'p':
			transfer = ASCII_TYPE;
			sockmode(s, SOCK_ASCII);
			dbf_fields(s, file, ASCIITEXT);
			break;
		/* send a back modified selector string for a dbf search */
		case 'q':
			transfer = ASCII_TYPE;
			sockmode(s, SOCK_ASCII);
			for (i=0; i < strlen(file); i++)
			{
			  if (file[i] == '\t')
				file[i] = '$';
			}
			usprintf(s, "0Results of your search\tz%s\t%s\t70\n",
					   file, Hostname);
			break;
		/* do a search on a dbf file and send the results to the client */
		case 'z':
			transfer = ASCII_TYPE;
			sockmode(s, SOCK_ASCII);
			dbf_do_search(s, file, ASCIITEXT,"");
			break;
		default:
			transfer = ASCII_TYPE;
			break;
		}

	switch(type[0])
	{
	   case 's':
	   case 'p':      /* Don't send a file for search types */
	   case 'q':
	   case 'z':
	   case '2':
	   case '7':
		  break;
	   default:

		if (transfer == ASCII_TYPE)
		fp = fopen(file, READ_TEXT);
		 else
		fp = fopen(file, READ_BINARY);

		 if (fp != NULLFILE)
		 {
		 sendfile(fp, s, transfer, 0, NULL);
		 fclose(fp);
		 }
		 else
		/* If no ginfo file lets make one on the fly */
		if (type[0] == '1' ) {
			mkginfo(s,file);
		}
		else usprintf(s, "ERROR: Could not find requested file");

	}    /* switch */
	/* If text files don't end with a blank line */
	if ( (transfer==ASCII_TYPE) && (type[0] == '0') )
	   usprintf(s, "\n.\012");
	   else
	if ( (transfer==ASCII_TYPE) && ( (type[0] != '0') && (type[0] != '2') ) )
		usprintf(s, ".\012");
quit:
	close_s(s);
	GophUsers--;
	log(s, "close Gopher");
}


int g0(int argc, char *argv[], void *p)
{
	close_s(Sgopher);
	gopherlog("Gopher server stopped\n");
	Sgopher = -1;
	return 0;
}

char *Message()
{
	static char str[100];
	char *p, *getenv();

		if ((p = getenv("MESSAGE")) == NULL)
			sprintf(str,"Have a nice day!");
		else
			sprintf(str,"%s", p);
	return(str);
}

char *City()
{
	static char str[50];
	char *p, *getenv();

		if ((p = getenv("CITY")) == NULL)
			sprintf(str,"this location");
		else
			sprintf(str,"%s", p);
	return(str);
}

char *
ftime(t)
long *t;
{
	/* Print out the time and date field as
	 *              "DAY day MONTH year hh:mm:ss ZONE"
	 */
	register struct tm *ltm;
	static char tz[4];
	static char str[40];
	char *p, *getenv();
	/* Read the system time */
	ltm = localtime(t);

	if (*tz == '\0')
		if ((p = getenv("TZ")) == NULL)
			strcpy(tz,"UTC");
		else
			strncpy(tz,p,3);

	/* format Sunday, January 01 '01, 23:59:59 EST*/
	sprintf(str,"%s, %s %.2d '%.02d, %02d:%02d:%02d %.3s\n",
		Day[ltm->tm_wday],
		Month[ltm->tm_mon],
		ltm->tm_mday,
		ltm->tm_year,
		ltm->tm_hour,
		ltm->tm_min,
		ltm->tm_sec,
		tz);
	return(str);
}

char *TheTime()
{
   char *cp;
   long t;

   time(&t);
   cp = ctime(&t);
   rip(cp);
   return cp;
}

int searchdir(int s, char *dir, char *pattern)
/*
   Search a directory for files containig 'pattern'.
   Output the results on socket s.
*/
{
	struct ffblk SR;
	int done;
	static int result;
	char buf[100], D[67], N[13];
	char *p, tmp[13];

	D[0] = '\0';
	N[0] = '\0';
	slashfix(dir);
	fsplit(dir, D, N);
	if (strlen(N) == 0)
	  strcpy(N, "*.*");

	strcpy(buf, dir);
	if (buf[strlen(buf)-1] == '/')
	  strcat(buf, N);

	if (strlen(pattern) <3)
	{
	  gnamefix(buf);
	  usprintf(s,"0'%s' is too short (minimum search string is 3 characters)\t0bogus\tbogus.host\t70\n7Repeat search with a longer search string\t7%s\t%s\t70\n", pattern, buf, Hostname);
	  return 0;
	}
	else
	if ((matchmake(pattern)) == 1)
	{
	  gnamefix(buf);
	  usprintf(s,"0'%s' is not a permitted search string\t0bogus\tbogus.host\t70\n7Repeat search with another search string\t7%s\t%s\t70\n", pattern, buf, Hostname);
	  return 0;
	}
	else
	done = findfirst(buf, &SR, FA_ARCH);
	while (!done)
	{
	  strcpy(buf, D);
	  strcat(buf, SR.ff_name);
	  slashfix(buf);
	if (((result = search(buf, pattern))==1))
	{
	gnamefix(buf);
	p = strrchr(buf, '/');
	strcpy(tmp, ++p);
	usprintf(s, "0%s\t0%s\t%s\t70\n", p, buf, Hostname);
	}
	  done = findnext(&SR);
	  }
	if (RO_SUBDIR == 1)
	{
	  strcpy(buf, dir);
	  strcat(buf, N);
	  done = findfirst(buf, &SR, FA_DIREC);
	  while (!done)
	  {
	if ((SR.ff_attrib & FA_DIREC) && (SR.ff_name[1] != '.'))
	{
	  strcpy(buf, dir);
	  strcat(buf, SR.ff_name);
	  searchdir(s, buf, pattern);
	}
	done = findnext(&SR);
	  }
	}
	return result;
}


int matchmake(char *pattern)
{
  FILE *WorkFile;
  char Line[80];
  int result;

  if((WorkFile = fopen(WORDSFILE, "rt")) != NULLFILE){
	while (!feof(WorkFile))
	{
	fgets(Line, 80, WorkFile);
	  strupr(Line);
	  strupr(pattern);
	if (strstr(Line, pattern) != NULL)
	  {
	result = 1;
	break;
	  }
	}
  fclose(WorkFile);
  }
  else result = 0;
  return(result);
}

int searchdirfile(int s, char *file, char *pattern)
/*
  Search a file for lines that match a particular pattern
  Return the result on socket s
*/
{
	FILE *f;
	char *p, Line[240], upperLine[240];
	int result;

	strupr(pattern);

	if (strlen(pattern) <3)
	{
	  gnamefix(file);
	  usprintf(s,"0'%s' is too short (minimum search string is 3 characters)\t0bogus\tbogus.host\t70\n7Repeat search with a longer search string\ts%s\t%s\t70\n", pattern, file, Hostname);
	  return 0;
	}
	else
	if ((dirmatchmake(pattern)) == 1)
	{
	  gnamefix(file);
	  usprintf(s,"0'%s' is not a permitted search string\t0bogus\tbogus.host\t70\n7Repeat search with another search string\ts%s\t%s\t70\n", pattern, file, Hostname);
	  return 0;
	}
	else
	if ((f = fopen(file, "rt")) != NULLFILE)  {
	while (!(feof(f)))  {
	fgets(Line, 240, f);
	strncpy(upperLine, Line, 240);
	p = strtok(upperLine, "\t");
	strupr(p);
	if (strstr(p, pattern) != NULL)
	  {
	   usprintf(s, "%s", Line);
	   result = 1;
	   continue;
	 }
	 }
  fclose(f);
  }
  else result = 0;
  return(result);
}


int dirmatchmake(char *pattern)
{
  FILE *WorkFile;
  char Line[80];
  int result;

  if((WorkFile = fopen(DIRBAD, "rt")) != NULLFILE){
	while (!feof(WorkFile))
	{
	fgets(Line, 80, WorkFile);
	strupr(pattern);
	strupr(Line);
	if (strstr(Line, pattern) != NULL)
	  {
	result = 1;
	break;
	  }
	}
  fclose(WorkFile);
  }
  else result = 0;
  return(result);
}


int search(char *filename, char *pattern)
/*
  Do text file searches for substrings.
  Returns TRUE if the file contains 'pattern', false otherwise.
*/
{
  FILE *TextFile;
  char Line[80];
  int result;

  if((TextFile = fopen(filename, "rt")) != NULLFILE){
  while (!feof(TextFile))
  {
	fgets(Line, 80, TextFile);
	strupr(Line);
	strupr(pattern);
	if (strstr(Line, pattern) != NULL)
	{
	  result = 1;
	  break;
	}
  }
  fclose(TextFile);
  }
  else result = 0;
  return(result);
}


int fsplit(char *P, char *D, char *F)
/*
  Split a file path into its directory and filename parts.
*/
{
  char *p;

  strcpy(D, P);
  p = strrchr(D, '/');
  if (!p)
	p = strrchr(D, '/');
  if (p)
  {
	 p++;
	 strcpy(F, p);
	 *p = '\0';
  }
  return 0;
}


int slashfix(char *s)
/*
  Converts backslashes ('\') to forward slashes ('/').
*/
{
  int i;

  for (i=0; i<strlen(s); i++)
	if (s[i] == '\\')
	   s[i] = '/';
  return 0;
}

int revslashfix(char *s)
/*
  Converts forwardslashes ('/') to back slashes ('\').
*/
{
  int i;

  for (i=0; i<strlen(s); i++)
	if (s[i] == '/')
	   s[i] = '\\';
  return 0;
}
int gnamefix(char *s)
/*
  Get rid of '/gopher' from a file path string.
*/
{
  char *p;
  char tmp[79];

  strupr(s);
  if ((p = strstr(s, "/GOPHER")))
  {
	 *p = '\0';
	 p+=7;
	 strcpy(tmp, s);
	 strcat(tmp, p);
	 strcpy(s, tmp);
  }
  return 0;
}


static int gopherlog(char *fmt,...)
/*
   Write stuff to the gopher log file.
   File is opened only when necessary.
   Now writes to /spool/gopher.log
*/
{
   va_list args;
   int len;
   FILE *f;

   f = fopen(GopherLog, APPEND_TEXT);
   va_start(args,fmt);
   len = vfprintf(f, fmt, args);
   va_end(args);
   fclose(f);

   return len;
}

#ifdef MAILLOG
int mail_logfile(char *user, char *host)
{
	struct sockaddr_in sock;
	char *cp;
	int s,i, reply;
	struct mbuf *bp;
	FILE *f;
	char buf[512];

	sock.sin_family = AF_INET;
	sock.sin_port = IPPORT_SMTP;

	if((sock.sin_addr.s_addr = resolve(host)) == 0)
	{
		tprintf("Gopher Log Mail Host %s unknown\n", host);
		return -1;
	}

	if((s = socket(AF_INET,SOCK_STREAM,0)) == -1)
	{
		tprintf("Can't create socket\n");
		return -1;
	}
	sockmode(s,SOCK_ASCII);
	if(connect(s,(char *)&sock,sizeof(sock)) == -1)
	{
		cp = sockerr(s);
		tprintf(" -- Connect failed: %s\n",cp != NULLCHAR ? cp : "");
		close_s(s);
		return -1;
	}

	recvline(s, buf, sizeof(buf));

	sprintf(buf, "HELO %s\n", Hostname);
	usprintf(s, "%s", buf);

	recvline(s, buf, sizeof(buf));
	sscanf(buf, "%d", &reply);
	if (reply != 250)
	{
	   tprintf("%s\n", buf);
	   return -1;
	}

	sprintf(buf, "MAIL FROM: <gopher@%s>\n", Hostname);
	usprintf(s, "%s", buf);

	recvline(s, buf, sizeof(buf));
	sscanf(buf, "%d", &reply);
	if (reply != 250)
	{
	   tprintf("%s\n", buf);
	   return -1;
	}

	sprintf(buf, "RCPT TO: <%s@%s>\n", user, host);
	usprintf(s, "%s", buf);

	recvline(s, buf, sizeof(buf));
	sscanf(buf, "%d", &reply);
	if (reply != 250)
	{
	   tprintf("%s\n", buf);
	   return -1;
	}

	sprintf(buf, "DATA\n");
	usprintf(s, "%s", buf);

	recvline(s, buf, sizeof(buf));
	sscanf(buf, "%d", &reply);
	if (reply != 354)
	{
	   tprintf("%s\n", buf);
	   return -1;
	}

	sprintf(buf, "To: %s@%s\n", user, host);
	usprintf(s, "%s", buf);
	sprintf(buf, "From: gopher@%s\n", Hostname);
	usprintf(s, "%s", buf);

	f = fopen(GopherLog, READ_TEXT);
	while (!feof(f))
	{
		fgets(buf, sizeof(buf), f);
		usprintf(s, "%s", buf);
		pwait(NULL);
	}
	fclose(f);

	usprintf(s, "\n.\n");
	sprintf(buf, "BYE\n");
	usprintf(s, "%s", buf);

	close_s(s);

	remove(GopherLog);
	f = fopen(GopherLog, APPEND_TEXT);
	fclose(f);

	return 0;
}


int sendgopherlog (int argc, char *argv[], void *p)
{
	char *pp, *user, *host;

	if (argc != 2)
	{
	  tprintf("Invalid number of args %d\n", argc);
	  return -1;
	}

	user = argv[1];
	pp = strrchr(argv[1], '@');
	*pp = '\0';
	pp++;
	host = pp;
	mail_logfile(user, host);

	return 0;
}
#endif /* ifdef MAILLOG */

int mkginfo(int s, char *file)
{
	FILE *fp;
	char dirstring[100];
	char file1[150];
	char *p,*ext;
	int length;
	int i, tt, done, doit;
	char type[1];
	char transfer[1]; /* Transfer type, either ascii or image */
	struct ffblk ffblk;
	char temp[100];
	char drive[3];
	drive[0]=file[0];
	drive[1]=file[1];
	drive[2]='\0';
	type[1]='\0';
	transfer[1]='\0';
	file1[0]='\0';

	strrev(file);
	strcpy(dirstring,strstr(file,"/"));
	strrev(dirstring);
	revslashfix(dirstring);

	done=findfirst(strcat(dirstring,"*.*"),&ffblk,FA_DIREC);

	if ( !done ) {
	strtok(dirstring,"*");
	/* get rid of /gopher part */
	p=&dirstring;
	p+=9;
	slashfix(p);
	}

		while (!done) {
		doit=1;
		/* Get the file extension */
		ext=(strstr(ffblk.ff_name,"."));
		if ( (!strcmpi(ext,".zip")) || (!strcmpi(ext,".arc")) || (!strcmpi(ext,".arj"))
			|| (!strcmpi(ext,".lha")) || (!strcmpi(ext,".zoo")) )
		 {
		 type[0]='5';           /* DOS binary archive only */
		 transfer[0]='9';
		 }
		 else if ( (!strcmpi(ext,".exe")) || (!strcmpi(ext,".com")) || (!strcmpi(ext,".ovr"))
			|| (!strcmpi(ext,".dll")) || (!strcmpi(ext,".wp"))  || (!strcmpi(ext,".z"))
			|| (!strcmpi(ext,".wb1")) || (!strcmpi(ext,".msw")) || (!strcmpi(ext,".taz"))
			|| (!strcmpi(ext,".hlp")) || (!strcmpi(ext,".ovl")) || (!strcmpi(ext,".tgz"))
			|| (!strcmpi(ext,".tar")) || (!strcmpi(ext,".wri")) || (!strcmpi(ext,".doc")) )
		 {
		 type[0]='9';           /* All other binary file types */
		 transfer[0]='9';
		 }
		 else if (!strcmpi(ext,".hqx"))
		 {
		 type[0]='4';           /* Mac BinHex file */
		 transfer[0]='0';
		 }
		 else if (!strcmpi(ext,".uue"))
		 {
		 type[0]='6';           /* Unix UUencoded file */
		 transfer[0]='0';
		 }
		 else if ( (!strcmpi(ext,".bmp")) || (!strcmpi(ext,".jpg")) || (!strcmpi(ext,".icn"))
			|| (!strcmpi(ext,".pcx")) || (!strcmpi(ext,".xbm")) || (!strcmpi(ext,".pic"))
			|| (!strcmpi(ext,".wpg")) || (!strcmpi(ext,".tif")) || (!strcmpi(ext,".pbm"))
			|| (!strcmpi(ext,".jif")) || (!strcmpi(ext,".gif")) || (!strcmpi(ext,".ico"))
			|| (!strcmpi(ext,".eps")) || (!strcmpi(ext,".rle")) )
		 {
		 type[0]='I';           /* All image files */
		 transfer[0]='9';
		 }
		 else if ( (!strcmpi(ext,".qt")) || (!strcmpi(ext,".dl")) || (!strcmpi(ext,".gl"))
			|| (!strcmpi(ext,".mpg")) || (!strcmpi(ext,".fli")) || (!strcmpi(ext,".qtw")) )
		 {
		 type[0]=';';           /* movies */
		 transfer[0]='9';
		 }
		 else if ( (!strcmpi(ext,".au")) || (!strcmpi(ext,".wav"))
			|| (!strcmpi(ext,".snd")) || (!strcmpi(ext,".sou")) )
		 {
		 type[0]='s';           /* Sound files */
		 transfer[0]='9';
		 }
		 else if (!strcmpi(ext,".dbf"))
		 {
		 type[0]='1';           /* database files */
		 transfer[0]='p';
		 }
		 else if (!strcmpi(ext,".cat")) 
		 {
		 doit=0;
		 }
		 else if ( ffblk.ff_attrib == FA_DIREC )
		 {
			 if ((!strcmpi(ffblk.ff_name,".")) || ( !strcmpi(ffblk.ff_name,"..")) )
			 {
			 doit=0;
			 }
			 else
			 {
			 type[0]='1';
			 transfer[0]='1';
			 }
		 }
		 else
		 {
		type[0]='0';
		transfer[0]='0';
		}
		if (doit==1)
		{
		if (transfer[0]=='p') usprintf(s, "%sSearch %s\t%s%s%s%s\t%s\t70\012",type, ffblk.ff_name,transfer,drive,p,ffblk.ff_name, Hostname);
		else if (transfer[0]=='1') usprintf(s, "%s%s\t%s%s%s%s\t%s\t70\012",type, strlwr(ffblk.ff_name),transfer,drive,p,ffblk.ff_name, Hostname);
		else usprintf(s, "%s%-17s[%ld bytes  %d-%d-%d]\t%s%s%s%s\t%s\t70\012",type, strlwr(ffblk.ff_name),ffblk.ff_fsize,
		(ffblk.ff_fdate >> 5) & 0xf,/* month */
		(ffblk.ff_fdate ) & 0x1f,/* day */
		(ffblk.ff_fdate >> 9) + 80/* year */
		,transfer,drive,p,ffblk.ff_name, Hostname);
		}
		done=findnext(&ffblk);
		 }
	strcat(file1, strcat(dirstring,"menu.cat"));
	fp = fopen(file1, READ_TEXT);
	if (fp != NULLFILE)
	{
	sendfile(fp, s, 0, 0, NULL);
	fclose(fp);
	}
	return 0;
}

