#include "global.h"
#include "config.h"
#include "cmdparse.h"

#ifdef TIPMAIL
int32 Mbtipperms = 0x109401;	/* no_slip, ftp_read, no_sendcmd, no_3party, no_convers */
#endif
int32 Mbax25perms = 0x109401;	/* no_slip, ftp_read, no_sendcmd, no_3party, no_convers */
int MBSecure = 0;
int AMPRonly = 0;


extern void usercvt __ARGS((void));
int dosecurity __ARGS((int argc,char *argv[],void *p));
#if 0
static int doftpgid __ARGS((int argc,char *argv[],void *p));
static int doftpuid __ARGS((int argc,char *argv[],void *p));
#endif
static int doax25security __ARGS((int argc,char *argv[],void *p));
static int dotipsecurity __ARGS((int argc,char *argv[],void *p));
static int doampronly __ARGS((int argc,char *argv[],void *p));
static int dombsecure __ARGS((int argc,char *argv[],void *p));
static int dousercvt __ARGS((int argc,char *argv[],void *p));
static int dolevel __ARGS((int argc,char *argv[],void *p));
static void displaylevel __ARGS((int index));
char *getsecurity __ARGS((char *name, int32 *perms));


/* security subcommand table */
static struct cmds DFAR SECtab[] = {
	"ampronly",	doampronly,	0, 0, NULLCHAR,
	"ax25perms",	doax25security,	0, 0, NULLCHAR,
	"encode",	dousercvt,	0, 0, NULLCHAR,
#if 0
	"ftpgid",	doftpgid,	0, 0, NULLCHAR,
	"ftpuid",	doftpuid,	0, 0, NULLCHAR,
#endif
	"level",	dolevel,	0, 0, NULLCHAR,
	"mbsecure",	dombsecure,	0, 0, NULLCHAR,
#if defined(TIPMAIL) && defined (ALLSERV)
	"tipperms",	dotipsecurity,	0, 0, NULLCHAR,
#endif
	NULLCHAR,
};

struct securitylevel {
	char *name;
	char *path;
	int32 perms;
};

#define MAXLEVELS 10

struct securitylevel seclevel[MAXLEVELS];


static void
displaylevel (int index)
{
	tprintf ("%-20.20s %-32.32s 0x%08lx\n", seclevel[index].name, seclevel[index].path, seclevel[index].perms);
}


char *
getsecurity (char *name, int32 *perms)
{
int k;

	for (k = 0; k < MAXLEVELS; k++)	{
		if (seclevel[k].name)
			if (!stricmp (name, seclevel[k].name))	{
				*perms = seclevel[k].perms;
				return (seclevel[k].path);
			}
	}
	return (NULLCHAR);
}


static int
dolevel(argc,argv,p)
int argc;
char *argv[];
void *p;
{
int k;

	if (argc == 1)	{
		tprintf("%-20.20s %-32.32s %s\n", "Security level", "Directory paths", "Permissions");
		for (k = 0; k < MAXLEVELS; k++)	{
			if (seclevel[k].name != NULLCHAR)
				displaylevel (k);
		}
		return 0;
	}
	if (argc < 4)	{
		tprintf ("syntax: security level levelname pathstring permissions\n");
		return 0;
	}
	for (k = 0; k < MAXLEVELS; k++)	{
		if ((seclevel[k].name == NULLCHAR) || !stricmp(seclevel[k].name, argv[1]))
			break;
	}
	if (k == MAXLEVELS)	{
		tprintf ("Can't add another security level. Already have the maximum of %d!\n", MAXLEVELS);
		return 0;
	}
	if (seclevel[k].name == NULLCHAR)
		seclevel[k].name = strdup(argv[1]);
	if (seclevel[k].path != NULLCHAR)
		free (seclevel[k].path);
	seclevel[k].path = strdup(argv[2]);
	seclevel[k].perms = (!strnicmp (argv[3], "0x", 2)) ? htol(argv[3]) : atol(argv[3]);
/*	displaylevel(k);	*/
	return 0;	
}

static int
dousercvt(argc,argv,p)
int argc;
char *argv[];
void *p;
{
	usercvt();
	tprintf("Password encoding complete!\n");
	return 0;
}

static int
dombsecure(argc,argv,p)
int argc;
char *argv[];
void *p;
{
	return setbool(&MBSecure,"Mailbox gateway RF secure flag",argc,argv);
}

static int
doampronly(argc,argv,p)
int argc;
char *argv[];
void *p;
{
	return setbool(&AMPRonly,"Mailbox gateway non-AMPR secure flag",argc,argv);
}

#if defined(TIPMAIL) && defined(ALLSERV)
static int
dotipsecurity(argc,argv,p)
int argc;
char *argv[];
void *p;
{
	return setlong(&Mbtipperms,"Tip security permission (for anonymous)",argc,argv);
}
#endif


static int
doax25security(argc,argv,p)
int argc;
char *argv[];
void *p;
{
	return setlong(&Mbax25perms,"AX25 security permission (for anonymous)",argc,argv);
}

#if 0
long SecureFTPgid, SecureFTPuid;

static int
doftpgid(argc,argv,p)
int argc;
char *argv[];
void *p;
{
	return setlong(&SecureFTPgid,"Non-sysop FTP group ID",argc,argv);
}

static int
doftpuid(argc,argv,p)
int argc;
char *argv[];
void *p;
{
	return setlong(&SecureFTPuid,"Non-sysop FTP user ID",argc,argv);
}
#endif


int
dosecurity(argc,argv,p)
int argc;
char *argv[];
void *p;
{
	return subcmd(SECtab,argc,argv,p);
}


