#include "global.h"

#include <sys/types.h>

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "mbuf.h"
#include "netuser.h"
#include "tcp.h"
#include "usock.h"
#include "socket.h"
#include "sockaddr.h"
#include "bldsaddr.h"
#include "commands.h"
char *tcp_port_name (int portnum);
int tcpgate1(int argc, char *argv[], void *p);
int tcpgate0(int argc, char *argv[], void *p);
static void tcpgateserv(int s,void *ds,void *p);
void gate_output(int s1,void *s2,void *p);

#ifdef TCPGATE
struct dest {
  int port;
  int socket;
  char *name;
  struct dest *next;
};

static struct dest *dests;


int
tcpgate0(argc,argv,p)
int argc;
char *argv[];
void *p;
{
struct dest *dp, *dp2;
int port;

	port = atoip(argv[1]);
	dp = dests;
	if (dp->port == port)	{
		dests = dp->next;
	} else	{
		for ( ; dp && dp->next->port != port; dp = dp->next) ;
		if (!dp || !dp->next)	{
			tprintf ("The tcpgate server is not active on port %d - nothing to stop!\n", port);
			return 0;
		}
		dp2 = dp->next;
		dp->next = dp2->next;
		dp = dp2;
	}

	close_s (dp->socket);
	if (dp->name)
		free (dp->name);
	free(dp);
	tprintf ("The tcpgate server removed from port %d\n", port);
	return 0;
}


int tcpgate1(int argc, char *argv[], void *p)
{
char buf[80];
int port;
char *name;
struct dest *dp;

	port = atoip(argv[1]);
#if 0
	if (argc < 3)
		sprintf(name = buf, "loopback:%d", port);
	else
#endif
		name = argv[2];
	for (dp = dests; dp && dp->port != port; dp = dp->next) ;
	if (!dp) {
		dp = (struct dest *) malloc(sizeof(struct dest));
		dp->port = port;
		dp->name = 0;
		dp->next = dests;
		dests = dp;
	}
	if (dp->name) free(dp->name);
	dp->name = strdup(name);
	dp->socket = -1;

	sprintf (buf, "TCPGATE %d listener", port);
	return (installserver (argc, argv, &dp->socket, buf, port,
		"TCPGATE server", tcpgateserv, 512, dp));
}

static void
tcpgateserv(s,ds,p)
int s;
void *ds;
void *p;
{
struct dest *dp = (struct dest *)ds;
struct mbuf *bp;
int addrlen;
struct sockaddr *addr = 0;
int rmtsock;
struct proc *rxproc;
struct sockaddr_in lsocket;

	log(s, "open TCPGATE server on port %s", dp->port);
	lsocket.sin_family = AF_INET;
	lsocket.sin_addr.s_addr = INADDR_ANY;
	lsocket.sin_port = dp->port;
	if (!(addr = build_sockaddr(dp->name, &addrlen)) ||
		(rmtsock = socket(addr->sa_family, SOCK_STREAM, 0)) <= 0 ||
		bind(rmtsock,(char *)&lsocket,sizeof(lsocket)) < 0 ||
		connect(rmtsock, (char *)addr, addrlen))
			return;

	/* Fork off the transmit process */
	rxproc = newproc("tcpgate_out",1024,gate_output,s,(void *)rmtsock,&rxproc,0);

	/* Process input on the connection */
	while(1)	{
		if(socklen(s,0))	{
			recv_mbuf(s,&bp,0,NULLCHAR,0);
			if(send_mbuf(rmtsock,bp,0,NULLCHAR,0) == -1)
				break;
		}
		if (rxproc == NULLPROC)
			break;
		mspause (200);
	}
	killproc(rxproc);
	close_s(s);
	close_s(rmtsock);
	s = -1;
}


/* User rlogin output task, started by user rlogin command */
void
gate_output(s1,s2,rx)
int s1;
void *s2;
void *rx;
{
struct mbuf *bp;
int sock2;
struct proc **rxproc;

	sock2 = (int)s2;
	rxproc = (struct proc **)rx;

	while(recv_mbuf(sock2,&bp,0,NULLCHAR,0) > 0){
		if(send_mbuf(s1,bp,0,NULLCHAR,0) == -1)
		break;
	}
	*rxproc = NULLPROC;
}

#endif /* TCPGATE */
